#-----------------------------------------------------------------------
# File    : bayes.mak
# Contents: commands to build naive and full Bayes classifier programs
# Author  : Christian Borgelt
# History : 2003.01.26 file created
#           2003.04.26 program bcdb added
#           2006.07.20 adapted to Visual Studio 8
#           2011.08.22 external module random added (from util/src)
#           2016.04.20 completed dependencies on header files
#-----------------------------------------------------------------------
THISDIR  = ..\..\bayes\src
UTILDIR  = ..\..\util\src
TABLEDIR = ..\..\table\src

CC       = cl.exe
DEFS     = /D WIN32 /D NDEBUG /D _CONSOLE /D _CRT_SECURE_NO_WARNINGS
CFLAGS   = /nologo /W3 /O2 /GS- $(DEFS) /c
INCS     = /I $(UTILDIR) /I $(TABLEDIR)

LD       = link.exe
LDFLAGS  = /nologo /subsystem:console /incremental:no
LIBS     = 

HDRS_1   = $(UTILDIR)\fntypes.h    $(UTILDIR)\scanner.h    \
           $(TABLEDIR)\attset.h    $(TABLEDIR)\table.h
HDRS_2   = $(HDRS_1)               $(TABLEDIR)\attmap.h mvnorm.h
HDRS     = $(HDRS_2)               $(UTILDIR)\tabread.h    \
           $(UTILDIR)\tabwrite.h   $(UTILDIR)\params.h     \
           $(UTILDIR)\error.h      nbayes.h fbayes.h
BCHDRS   = $(HDRS) mvnorm.h fbayes.h nbayes.h
OBJS     = $(UTILDIR)\arrays.obj   $(UTILDIR)\escape.obj   \
           $(UTILDIR)\tabread.obj  $(UTILDIR)\tabwrite.obj \
           $(UTILDIR)\scanner.obj  $(UTILDIR)\random.obj   \
           $(TABLEDIR)\attset1.obj $(TABLEDIR)\attset2.obj \
           $(TABLEDIR)\attset3.obj $(TABLEDIR)\attmap.obj  \
           $(TABLEDIR)\table1.obj  $(TABLEDIR)\tab2rw.obj
BCI_O    = $(OBJS) $(UTILDIR)\params.obj \
           mvnorm.obj fbc_ind.obj nbc_ind.obj bci.obj
BCX_O    = $(OBJS) $(UTILDIR)\params.obj \
           mvn_pars.obj fbc_exec.obj nbc_exec.obj bcx.obj
BCDB_O   = $(OBJS) $(UTILDIR)\params.obj \
           mvn_pars.obj fbc_exec.obj nbc_exec.obj bcdb.obj
CORR_O   = $(UTILDIR)\escape.obj   $(UTILDIR)\tabread.obj  \
           $(UTILDIR)\symtab.obj   $(UTILDIR)\random.obj   \
           mvnorm.obj corr.obj
PRGS     = bci.exe bcx.exe bcdb.exe corr.exe

#-----------------------------------------------------------------------
# Build Programs
#-----------------------------------------------------------------------
all:          $(PRGS)

bci.exe:      $(BCI_O) bayes.mak
	$(LD) $(LDFLAGS) $(BCI_O) $(LIBS) /out:$@

bcx.exe:      $(BCX_O) bayes.mak
	$(LD) $(LDFLAGS) $(BCX_O) $(LIBS) /out:$@

bcdb.exe:     $(BCDB_O) bayes.mak
	$(LD) $(LDFLAGS) $(BCDB_O) $(LIBS) /out:$@

corr.exe:     $(CORR_O) bayes.mak
	$(LD) $(LDFLAGS) $(CORR_O) $(LIBS) /out:$@

#-----------------------------------------------------------------------
# Main Programs
#-----------------------------------------------------------------------
bci.obj:      $(BCHDRS) $(UTILDIR)\arrays.h
bci.obj:      bci.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) bci.c /Fo$@

bcx.obj:      $(BCHDRS)
bcx.obj:      bcx.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) bcx.c /Fo$@

bcdb.obj:     $(BCHDRS)
bcdb.obj:     $(UTILDIR)\random.h bcdb.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) bcdb.c /Fo$@

corr.obj:     $(UTILDIR)\fntypes.h  $(UTILDIR)\arrays.h \
              $(UTILDIR)\tabread.h  $(UTILDIR)\symtab.h \
              $(UTILDIR)\error.h    mvnorm.h
corr.obj:     corr.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) corr.c /Fo$@

#-----------------------------------------------------------------------
# Naive Bayes Classifier Management
#-----------------------------------------------------------------------
nbc_ind.obj:  $(HDRS_1)
nbc_ind.obj:  nbayes.h nbayes.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) /D NBC_INDUCE nbayes.c /Fo$@

nbc_exec.obj: $(HDRS_1)
nbc_exec.obj: nbayes.h nbayes.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) /D NBC_PARSE nbayes.c /Fo$@

#-----------------------------------------------------------------------
# Full Bayes Classifier Management
#-----------------------------------------------------------------------
fbc_ind.obj:  $(HDRS_2)
fbc_ind.obj:  fbayes.h fbayes.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) /D FBC_INDUCE fbayes.c /Fo$@

fbc_exec.obj: $(HDRS_2)
fbc_exec.obj: fbayes.h fbayes.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) /D FBC_PARSE fbayes.c /Fo$@

#-----------------------------------------------------------------------
# Multivariate Normal Distribution Management
#-----------------------------------------------------------------------
mvnorm.obj:   mvnorm.h mvnorm.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) mvnorm.c /Fo$@

mvn_pars.obj: $(UTILDIR)\scanner.h mvnorm.h mvnorm.c bayes.mak
	$(CC) $(CFLAGS) $(INCS) /D MVN_PARSE mvnorm.c /Fo$@

#-----------------------------------------------------------------------
# External Modules
#-----------------------------------------------------------------------
$(UTILDIR)\arrays.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak arrays.obj
	cd $(THISDIR)
$(UTILDIR)\escape.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak escape.obj
	cd $(THISDIR)
$(UTILDIR)\symtab.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak symtab.obj
	cd $(THISDIR)
$(UTILDIR)\random.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak random.obj
	cd $(THISDIR)
$(UTILDIR)\tabread.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak tabread.obj
	cd $(THISDIR)
$(UTILDIR)\tabwrite.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak tabwrite.obj
	cd $(THISDIR)
$(UTILDIR)\scanner.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak scanner.obj
	cd $(THISDIR)
$(UTILDIR)\params.obj:
	cd $(UTILDIR)
	$(MAKE) /f util.mak params.obj
	cd $(THISDIR)
$(TABLEDIR)\attset1.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak attset1.obj
	cd $(THISDIR)
$(TABLEDIR)\attset2.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak attset2.obj
	cd $(THISDIR)
$(TABLEDIR)\attset3.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak attset3.obj
	cd $(THISDIR)
$(TABLEDIR)\attmap.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak attmap.obj
	cd $(THISDIR)
$(TABLEDIR)\table1.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak table1.obj
	cd $(THISDIR)
$(TABLEDIR)\tab2rw.obj:
	cd $(TABLEDIR)
	$(MAKE) /f table.mak tab2rw.obj
	cd $(THISDIR)

#-----------------------------------------------------------------------
# Install
#-----------------------------------------------------------------------
install:
	-@copy *.exe ..\..\..\bin

#-----------------------------------------------------------------------
# Clean up
#-----------------------------------------------------------------------
localclean:
	-@erase /Q *~ *.obj *.idb *.pch $(PRGS)

clean:
	$(MAKE) /f bayes.mak localclean
	cd $(UTILDIR)
	$(MAKE) /f util.mak clean
	cd $(TABLEDIR)
	$(MAKE) /f table.mak localclean
	cd $(THISDIR)
